// src/index.ts
import { createTSLanguageService } from "../node_modules/@vtsls/typescript-lanuguage-service/dist/index.js";
import {
  ConfigurationRequest,
  createConnection,
  LogMessageNotification,
  ProposedFeatures,
  ShowDocumentRequest,
  ShowMessageRequest
} from "../node_modules/vscode-languageserver/node.js";
import { URI } from "../node_modules/vscode-uri/lib/umd/index.js";

// src/capabilities.ts
import {
  codeActionKinds,
  commands,
  completionTriggerCharacters,
  onTypeFormatFirstTriggerCharacter,
  onTypeFormatMoreTriggerCharacter,
  semanticTokenModifiers,
  semanticTokenTypes,
  signatureHelpReTriggerCharacters,
  signatureHelpTriggerCharacters
} from "../node_modules/@vtsls/typescript-lanuguage-service/dist/index.js";
import { TextDocumentSyncKind } from "../node_modules/vscode-languageserver/node.js";
function getTsLspDefaultCapabilities() {
  return {
    textDocumentSync: {
      openClose: true,
      change: TextDocumentSyncKind.Incremental,
      willSave: false,
      willSaveWaitUntil: false,
      save: false
    },
    completionProvider: {
      triggerCharacters: completionTriggerCharacters,
      resolveProvider: true,
      completionItem: {
        labelDetailsSupport: true
      }
    },
    hoverProvider: true,
    signatureHelpProvider: {
      triggerCharacters: signatureHelpTriggerCharacters,
      retriggerCharacters: signatureHelpReTriggerCharacters
    },
    declarationProvider: false,
    definitionProvider: true,
    typeDefinitionProvider: true,
    implementationProvider: true,
    referencesProvider: true,
    documentHighlightProvider: true,
    documentSymbolProvider: {
      label: "typescript"
    },
    codeActionProvider: {
      codeActionKinds,
      resolveProvider: true
    },
    codeLensProvider: { resolveProvider: true },
    documentLinkProvider: void 0,
    colorProvider: false,
    workspaceSymbolProvider: { resolveProvider: false },
    documentFormattingProvider: true,
    documentRangeFormattingProvider: true,
    documentOnTypeFormattingProvider: {
      firstTriggerCharacter: onTypeFormatFirstTriggerCharacter,
      moreTriggerCharacter: onTypeFormatMoreTriggerCharacter
    },
    renameProvider: {
      prepareProvider: true
    },
    foldingRangeProvider: true,
    selectionRangeProvider: true,
    executeCommandProvider: {
      commands
    },
    callHierarchyProvider: true,
    linkedEditingRangeProvider: false,
    semanticTokensProvider: {
      legend: {
        tokenTypes: semanticTokenTypes,
        tokenModifiers: semanticTokenModifiers
      },
      full: true,
      range: true
    },
    monikerProvider: false,
    typeHierarchyProvider: false,
    inlineValueProvider: false,
    inlayHintProvider: true,
    diagnosticProvider: {
      identifier: "typescript",
      interFileDependencies: true,
      workspaceDiagnostics: false
    },
    workspace: {
      workspaceFolders: {
        supported: true,
        changeNotifications: true
      },
      fileOperations: void 0
    }
  };
}

// src/index.ts
function createLanguageServer() {
  const conn = createConnection(ProposedFeatures.all);
  conn.onInitialize((params) => {
    return onServerInitialize(conn, params);
  });
  conn.listen();
}
function onServerInitialize(conn, params) {
  const clientCapabilities = params.capabilities;
  const root = params.rootUri ?? (params.rootPath ? URI.file(params.rootPath).toString() : void 0);
  const folders = params.workspaceFolders ?? (typeof root == "string" ? [{ name: root, uri: root }] : void 0);
  const service = createTSLanguageService({
    locale: params.locale,
    workspaceFolders: folders,
    clientCapabilities
  });
  async function initializeService() {
    void conn.sendRequest(ConfigurationRequest.type, { items: [{}] }).then((config) => {
      if (Array.isArray(config)) {
        void service.initialize(config[0]);
      }
    });
    try {
      await service.initialized.wait();
    } catch (e) {
      conn.dispose();
    }
  }
  conn.onInitialized(() => {
    void initializeService();
  });
  bindServiceHandlers(conn, service);
  return {
    capabilities: getTsLspDefaultCapabilities(),
    serverInfo: { name: "vtsls", version: "0.0.5" }
  };
}
function bindServiceHandlers(conn, service) {
  service.onLogMessage(
    (params) => void conn.sendNotification(LogMessageNotification.method, params)
  );
  service.onShowMessage((params) => conn.sendRequest(ShowMessageRequest.type, params));
  service.onShowDocument(
    async (params) => (await conn.sendRequest(ShowDocumentRequest.type, params)).success
  );
  service.onWorkDoneProgress(() => conn.window.createWorkDoneProgress());
  service.onApplyWorkspaceEdit((params) => conn.workspace.applyEdit(params));
  service.onDiagnostics((params) => conn.sendDiagnostics(params));
  conn.onExit(() => service.dispose());
  conn.onShutdown(() => service.dispose());
  conn.onDidOpenTextDocument(service.openTextDocument);
  conn.onDidCloseTextDocument(service.closeTextDocument);
  conn.onDidChangeTextDocument(service.changeTextDocument);
  conn.onDidChangeConfiguration(service.changeConfiguration);
  conn.workspace.onDidRenameFiles(service.renameFiles);
  conn.workspace.onDidChangeWorkspaceFolders((event) => service.changeWorkspaceFolders({ event }));
  conn.onCompletion(service.completion);
  conn.onCompletionResolve(service.completionItemResolve);
  conn.onDocumentHighlight(service.documentHighlight);
  conn.onSignatureHelp(service.signatureHelp);
  conn.onDefinition(service.definition);
  conn.onReferences(service.references);
  conn.onHover(service.hover);
  conn.onDocumentSymbol(service.documentSymbol);
  conn.onWorkspaceSymbol(service.workspaceSymbol);
  conn.onCodeAction(service.codeAction);
  conn.onCodeActionResolve(service.codeActionResolve);
  conn.onExecuteCommand(service.executeCommand);
  conn.onImplementation(service.implementation);
  conn.onTypeDefinition(service.typeDefinition);
  conn.onDocumentFormatting(service.documentFormatting);
  conn.onDocumentRangeFormatting(service.documentRangeFormatting);
  conn.onDocumentOnTypeFormatting(service.documentOnTypeFormatting);
  conn.onPrepareRename(service.prepareRename);
  conn.onRenameRequest(service.rename);
  conn.onFoldingRanges(service.foldingRanges);
  conn.onSelectionRanges(service.selectionRanges);
  conn.onCodeLens(service.codeLens);
  conn.onCodeLensResolve(service.codeLensResolve);
  conn.languages.callHierarchy.onPrepare(service.prepareCallHierachy);
  conn.languages.callHierarchy.onIncomingCalls(service.incomingCalls);
  conn.languages.callHierarchy.onOutgoingCalls(service.outgoingCalls);
  conn.languages.inlayHint.on(service.inlayHint);
  conn.languages.semanticTokens.on(service.semanticTokensFull);
  conn.languages.semanticTokens.onRange(service.semanticTokensRange);
}
createLanguageServer();
